@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2015 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2015 - 2021, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -466,7 +466,7 @@ void Rules::parseRuleSettings (
 ////////////////////////////////////////////////////////////////////////////////
 unsigned int Rules::getIndentation (const std::string& line)
 {
-  auto indent = line.find_first_not_of (" ");
+  auto indent = line.find_first_not_of (' ');
   if (indent == std::string::npos)
     indent = 0;
 
