@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2021, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -121,7 +121,7 @@ int CmdSummary (
     days_end = Datetime ();
   }
 
-  for (Datetime day = days_start; day < days_end; day++)
+  for (Datetime day = days_start.startOfDay (); day < days_end; ++day)
   {
     auto day_range = getFullDay (day);
     time_t daily_total = 0;
