@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2021, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 #include <algorithm>
-#include <deque>
+#include <cassert>
 
 #include <cmake.h>
 #include <shared.h>
@@ -126,35 +126,72 @@ std::vector <Range> getAllExclusions (
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void flattenDatabase (Database& database, const Rules& rules)
+// Potentially expand the latest interval into a collection of synthetic
+// intervals.
+std::vector <Interval> expandLatest (const Interval& latest, const Rules& rules)
 {
-  auto latest = getLatestInterval (database);
+  int current_id = 0;
+  std::vector <Interval> intervals;
+  // If the latest interval is open, check for synthetic intervals
   if (latest.is_open ())
   {
     auto exclusions = getAllExclusions (rules, {latest.start, Datetime ()});
     if (! exclusions.empty ())
     {
-      Interval modified {latest};
+      std::vector <Interval> flattened = flatten (latest, exclusions);
 
-      // Update database.
-      database.deleteInterval (latest);
-      for (auto& interval : flatten (modified, exclusions))
+      // If flatten() converted the latest interval into a group of synthetic
+      // intervals, the number of returned intervals will be greater than 1,
+      // otherwise, it just returned the non-synthetic latest interval.
+      if (flattened.size () > 1)
       {
-        database.addInterval (interval, rules.getBoolean ("verbose"));
+        std::reverse (flattened.begin (), flattened.end ());
+        for (auto interval : flattened)
+        {
+          interval.synthetic = true;
+          interval.id = ++current_id;
+          intervals.push_back (std::move (interval));
+        }
       }
     }
   }
+
+  if (intervals.empty ())
+  {
+    intervals.push_back (latest);
+    intervals.back().id = 1;
+  }
+  return intervals;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Convert what would be synthetic intervals into real intervals in the database
+void flattenDatabase (Database& database, const Rules& rules)
+{
+  Interval latest = getLatestInterval (database);
+  std::vector <Interval> expanded = expandLatest (latest, rules);
+
+  if (expanded.size () > 1)
+  {
+    bool verbose = rules.getBoolean ("verbose");
+    database.deleteInterval (latest);
+    for (auto it = expanded.rbegin (); it != expanded.rend (); ++it)
+    {
+      it->synthetic = false;
+      database.addInterval (*it, verbose);
+    }
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// This function will return synthetic intervals
+// Return collection of intervals (synthetic included) sorted by id
 std::vector <Interval> getIntervalsByIds (
   Database& database,
   const Rules& rules,
   const std::set <int>& ids)
 {
   std::vector <Interval> intervals;
-  std::deque <Interval> synthetic;
+  intervals.reserve (ids.size ());
 
   int current_id = 0;
   auto id_it = ids.begin ();
@@ -163,60 +200,35 @@ std::vector <Interval> getIntervalsByIds (
   auto it = database.begin ();
   auto end = database.end ();
 
-  Interval latest;
+  // Because the latest recorded interval may be expanded into synthetic
+  // intervals, we'll handle it specially
   if (it != end )
   {
-    latest = IntervalFactory::fromSerialization (*it);
-  }
+    Interval latest = IntervalFactory::fromSerialization (*it);
+    ++it;
 
-  // If the latest interval is open, check for synthetic intervals
-  if (latest.is_open ())
-  {
-    auto exclusions = getAllExclusions (rules, {latest.start, Datetime ()});
-    if (! exclusions.empty ())
+    for (auto& interval : expandLatest (latest, rules))
     {
-        // We're converting the latest interval into synthetic intervals so we need to skip it
-        ++it;
+      ++current_id;
 
-        for (auto& interval : flatten (latest, exclusions))
-        {
-          ++current_id;
-          interval.synthetic = true;
-          interval.id = current_id;
-          synthetic.push_front (interval);
-        }
+      if (id_it == id_end)
+      {
+        break;
+      }
+
+      if (interval.id == *id_it)
+      {
+        intervals.push_back (interval);
+        ++id_it;
+      }
     }
   }
 
-  intervals.reserve (ids.size ());
-
-  // First look for the ids in our set of synthetic intervals
-  for (auto& interval : synthetic)
-  {
-    if (id_it == id_end)
-    {
-      break;
-    }
-
-    if (interval.id == *id_it)
-    {
-      intervals.push_back (interval);
-      ++id_it;
-    }
-  }
-
-  current_id = synthetic.size ();
-
-  // We'll find remaining intervals from the database itself
-  for ( ; it != end; ++it)
+  // We'll find remaining intervals from the normal recorded intervals
+  for ( ; (it != end) && (id_it != id_end); ++it)
   {
     ++current_id;
 
-    if (id_it == id_end)
-    {
-      break;
-    }
-
     if (current_id == *id_it)
     {
       Interval interval = IntervalFactory::fromSerialization (*it);
@@ -235,23 +247,6 @@ std::vector <Interval> getIntervalsByIds (
   return intervals;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-std::vector <Interval> subset (
-  const Interval& filter,
-  const std::deque <Interval>& intervals)
-{
-  std::vector <Interval> all;
-  for (auto& interval : intervals)
-  {
-    if (matchesFilter (interval, filter))
-    {
-      all.push_back (interval);
-    }
-  }
-
-  return all;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 std::vector <Interval> subset (
   const Interval& filter,
@@ -486,91 +481,61 @@ Interval clip (const Interval& interval, const Range& range)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// Return collection of intervals that match the filter (synthetic intervals
+// included) sorted by date
 std::vector <Interval> getTracked (
   Database& database,
   const Rules& rules,
   Interval& filter)
 {
-  int id_skip = 0;
-  std::deque <Interval> intervals;
+  int current_id = 0;
+  std::vector <Interval> intervals;
 
-  for (auto& line : database)
+  auto it = database.begin ();
+  auto end = database.end ();
+
+  // Because the latest recorded interval may be expanded into synthetic
+  // intervals, we'll handle it specially
+  if (it != end )
   {
-    Interval interval = IntervalFactory::fromSerialization(line);
+    Interval latest = IntervalFactory::fromSerialization (*it);
+    ++it;
 
-    // Since we are moving backwards, and the intervals are in sorted order,
-    // if the filter is after the interval, we know there will be no more matches
-    if (matchesRange (interval, filter))
+    for (auto& interval : expandLatest (latest, rules))
     {
-      intervals.push_front (std::move (interval));
-    }
-    else if (interval.start.toEpoch () >= filter.start.toEpoch ())
-    {
-      ++id_skip;
-    }
-    else
-    {
-      break;
-    }
-  }
-
-  if (! intervals.empty ())
-  {
-    auto latest = intervals.back ();
-
-    if (latest.is_open ())
-    {
-      // Get the set of expanded exclusions that overlap the range defined by the open interval.
-      Interval exclusion_range {};
-      exclusion_range.start = latest.start;
-      exclusion_range.end = Datetime();
-
-      auto exclusions = getAllExclusions (rules, exclusion_range);
-      if (! exclusions.empty ())
+      ++current_id;
+      if (matchesFilter (interval, filter))
       {
-        intervals.pop_back ();
-
-        for (auto& interval : flatten (latest, exclusions))
-        {
-          if (latest.synthetic ||
-              latest != interval)
-            interval.synthetic = true;
-
-          intervals.push_back (interval);
-        }
+        interval.id = current_id;
+        intervals.push_back (interval);
       }
     }
   }
 
-  // Assign an ID to each interval before we prune ones that do not have matching tags
-  for (unsigned int i = 0; i < intervals.size (); ++i)
+  for (; it != end; ++it)
   {
-    intervals[i].id = intervals.size () - i + id_skip;
-  }
+    Interval interval = IntervalFactory::fromSerialization(*it);
+    interval.id = ++current_id;
 
-  // Now prune the intervals without matching tags
-  if (filter.tags ().size () > 0)
-  {
-    auto cmp = [&filter] (const Interval& interval)
-               {
-                 for (const auto& tag : filter.tags ())
-                 {
-                   if (! interval.hasTag (tag))
-                   {
-                     return true;
-                   }
-                 }
-                 return false;
-               };
-
-    intervals.erase (std::remove_if (intervals.begin (), intervals.end (), cmp),
-                     intervals.end ());
+    if (matchesFilter (interval, filter))
+    {
+      intervals.push_back (std::move (interval));
+    }
+    else if ((interval.start < filter.start) && ! interval.intersects (filter))
+    {
+      // Since we are moving backwards in time, and the intervals are in sorted
+      // order, if the filter is after the interval, we know there will be no
+      // more matches
+      break;
+    }
   }
 
   debug (format ("Loaded {1} tracked intervals", intervals.size ()));
 
-  return std::vector <Interval> (std::make_move_iterator (intervals.begin ()),
-                                 std::make_move_iterator (intervals.end ()));
+  // By default intervals are sorted by id, but getTracked needs to return the
+  // intervals sorted by date, which are ids in reverse order.
+  std::reverse (intervals.begin (), intervals.end ());
+  return intervals;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
