@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016, 2018 - 2020, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016, 2018 - 2021, Thomas Lauf, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -124,10 +124,7 @@ bool domGet (
 
         std::stringstream s;
 
-        for (const auto& tag : tags)
-        {
-          s << format ( "{1} ", tag );
-        }
+        s << joinQuotedIfNeeded ( " ", tags );
 
         value = s.str();
         return true;
@@ -196,18 +193,18 @@ bool domGet (
           return true;
         }
 
-        int n;
+        int m;
         // dom.tracked.<N>.tag.<M>
         if (pig.skipLiteral ("tag.") &&
-            pig.getDigits (n))
+            pig.getDigits (m))
         {
           std::vector <std::string> tags;
           for (auto& tag : tracked[count - n].tags ())
             tags.push_back (tag);
 
-          if (n <= static_cast <int> (tags.size ()))
+          if (m <= static_cast <int> (tags.size ()))
           {
-            value = format ("{1}", tags[n - 1]);
+            value = format ("{1}", tags[m - 1]);
             return true;
           }
         }
