@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2018, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -66,9 +66,15 @@ bool Datetime::isoEnabled            = true;
 bool Datetime::standaloneDateEnabled = true;
 bool Datetime::standaloneTimeEnabled = true;
 
-// When true, HH:MM:SS is assumed to be today if the time > now, otherwise it is
-// asumed to be tomorrow.  When false, it is always today.
-bool Datetime::timeRelative          = true;
+// When true,
+// - HH:MM:SS is assumed to be today if the time > now, otherwise it is assumed to be tomorrow.
+// - day name is converted to date relative to now
+// - 1st, 2nd, 3rd, ... is converted to date relative to now
+// When false,
+// - HH:MM:SS is always today.
+// - day name is always converted to date before now
+// - 1st, 2nd, 3rd, ... is always converted to date before now
+bool Datetime::timeRelative = true;
 
 ////////////////////////////////////////////////////////////////////////////////
 Datetime::Datetime ()
@@ -97,7 +103,7 @@ Datetime::Datetime (const time_t t)
 Datetime::Datetime (const int y, const int m, const int d)
 {
   // Protect against arguments being passed in the wrong order.
-  assert (y >= 1969 && y < 2100);
+  assert (y >= 1969 && y <= 9999);
   assert (m >= 1 && m <= 12);
   assert (d >= 1 && d <= 31);
 
@@ -118,7 +124,7 @@ Datetime::Datetime (const int y,  const int m,  const int d,
                     const int hr, const int mi, const int se)
 {
   // Protect against arguments being passed in the wrong order.
-  assert (y >= 1969 && y < 2100);
+  assert (y >= 1969 && y <= 9999);
   assert (m >= 1 && m <= 12);
   assert (d >= 1 && d <= 31);
   assert (hr >= 0 && hr <= 24);
@@ -556,8 +562,8 @@ bool Datetime::parse_formatted (Pig& pig, const std::string& format)
 //   <ordinal> 12th 2017-03-12T00:00:00
 //   <day> monday   2017-03-06T00:00:00
 //   <month> april  2017-04-01T00:00:00
-//   later          2038-01-18T00:00:00  Unaffected
-//   someday        2038-01-18T00:00:00  Unaffected
+//   later          9999-12-30T00:00:00  Unaffected
+//   someday        9999-12-30T00:00:00  Unaffected
 //   sopd           2017-03-04T00:00:00  Unaffected
 //   sod            2017-03-05T00:00:00  Unaffected
 //   sond           2017-03-06T00:00:00  Unaffected
@@ -593,7 +599,7 @@ bool Datetime::parse_formatted (Pig& pig, const std::string& format)
 //   sony           2018-01-01T00:00:00  Unaffected
 //   eopy           2017-01-01T00:00:00  Unaffected
 //   eoy            2018-01-01T00:00:00  Unaffected
-//   eony           2019-01-01T00:00:00  Unaffected
+//   eony           2020-01-01T00:00:00  Unaffected
 //   easter         2017-04-16T00:00:00
 //   eastermonday   2017-04-16T00:00:00
 //   ascension      2017-05-25T00:00:00
@@ -619,7 +625,7 @@ bool Datetime::parse_named (Pig& pig)
   }
 
 /*
-  // This grpoup contains "1st monday ..." which must be processed before
+  // This group contains "1st monday ..." which must be processed before
   // initializeOrdinal below.
   if (initializeNthDayInMonth (tokens))
   {
@@ -693,10 +699,11 @@ bool Datetime::parse_epoch (Pig& pig)
 {
   auto checkpoint = pig.cursor ();
 
-  int epoch {};
+  long long epoch {};
   if (pig.getDigits (epoch)             &&
       ! unicodeLatinAlpha (pig.peek ()) &&
-      epoch >= 315532800)
+      epoch >= 315532800                &&
+      epoch < 253402293599 )  // 9999-12-31, 23:59:59 AoE
   {
     _date = static_cast <time_t> (epoch);
     return true;
@@ -1425,7 +1432,7 @@ bool Datetime::initializeOrdinal (Pig& pig)
 {
   auto checkpoint = pig.cursor ();
 
-  int number = 0;
+  long long number = 0;
   if (pig.getDigits (number) &&
       number > 0             &&
       number <= 31)
@@ -1455,32 +1462,30 @@ bool Datetime::initializeOrdinal (Pig& pig)
         int m = t->tm_mon + 1;
         int d = t->tm_mday;
 
-        // If it is this month.
-        if (d < number &&
-            number <= daysInMonth (y, m))
-        {
-          t->tm_hour = t->tm_min = t->tm_sec = 0;
-          t->tm_mon  = m - 1;
-          t->tm_mday = number;
-          t->tm_year = y - 1900;
-          t->tm_isdst = -1;
-          _date = mktime (t);
-        }
-        else
+        if (timeRelative && (1 <= number && number <= d))
         {
           if (++m > 12)
           {
             m = 1;
             y++;
           }
-
-          t->tm_hour = t->tm_min = t->tm_sec = 0;
-          t->tm_mon  = m - 1;
-          t->tm_mday = number;
-          t->tm_year = y - 1900;
-          t->tm_isdst = -1;
-          _date = mktime (t);
         }
+        else if (!timeRelative && (d < number && number <= daysInMonth (y, m)))
+        {
+          if (--m < 1)
+          {
+            m = 12;
+            y--;
+          }
+        }
+
+        t->tm_hour = t->tm_min = t->tm_sec = 0;
+        t->tm_mon  = m - 1;
+        t->tm_mday = number;
+        t->tm_year = y - 1900;
+        t->tm_isdst = -1;
+
+        _date = mktime (t);
 
         return true;
       }
@@ -1513,9 +1518,13 @@ bool Datetime::initializeDayName (Pig& pig)
         struct tm* t = localtime (&now);
 
         if (t->tm_wday >= day)
-          t->tm_mday += day - t->tm_wday + 7;
+        {
+          t->tm_mday += day - t->tm_wday + (timeRelative ? 7 : 0);
+        }
         else
-          t->tm_mday += day - t->tm_wday;
+        {
+          t->tm_mday += day - t->tm_wday - (timeRelative ? 0 : 7);
+        }
 
         t->tm_hour = t->tm_min = t->tm_sec = 0;
         t->tm_isdst = -1;
@@ -1551,8 +1560,10 @@ bool Datetime::initializeMonthName (Pig& pig)
         time_t now = time (nullptr);
         struct tm* t = localtime (&now);
 
-        if (t->tm_mon >= month)
+        if (t->tm_mon >= month && timeRelative)
+        {
           t->tm_year++;
+        }
 
         t->tm_mon = month;
         t->tm_mday = 1;
@@ -1593,9 +1604,9 @@ bool Datetime::initializeLater (Pig& pig)
       struct tm* t = localtime (&now);
 
       t->tm_hour = t->tm_min = t->tm_sec = 0;
-      t->tm_year = 138;
-      t->tm_mon = 0;
-      t->tm_mday = 18;
+      t->tm_year = 8099;  // Year 9999
+      t->tm_mon = 11;
+      t->tm_mday = 30;
       t->tm_isdst = -1;
       _date = mktime (t);
       return true;
@@ -2802,7 +2813,7 @@ bool Datetime::initializeInformalTime (Pig& pig)
       hours = 10 * hours + digit;
 
     int minutes = 0;
-    int seconds = 0;
+    long long seconds = 0;
     if (pig.skip (':'))
     {
       if (! pig.getDigit2 (minutes))
@@ -3043,16 +3054,16 @@ bool Datetime::initializeNthDayInMonth (const std::vector <std::string>& tokens)
 bool Datetime::isOrdinal (const std::string& token, int& ordinal)
 {
   Pig p (token);
-  int number;
+  long long number;
   std::string suffix;
   if (p.getDigits (number) &&
       p.getRemainder (suffix))
   {
-    if (((number >= 11 || number <= 13) && suffix == "th") ||
-        (number % 10 == 1               && suffix == "st") ||
-        (number % 10 == 2               && suffix == "nd") ||
-        (number % 10 == 3               && suffix == "rd") ||
-        (                                  suffix == "th"))
+    if ((number >= 11 && number <= 13 && suffix == "th") ||
+        (number % 10 == 1             && suffix == "st") ||
+        (number % 10 == 2             && suffix == "nd") ||
+        (number % 10 == 3             && suffix == "rd") ||
+        (                                suffix == "th"))
     {
       ordinal = number;
       return true;
@@ -3067,7 +3078,7 @@ bool Datetime::isOrdinal (const std::string& token, int& ordinal)
 bool Datetime::validate ()
 {
   // _year;
-  if ((_year    && (_year    <   1900 || _year    >                                  2200)) ||
+  if ((_year    && (_year    <   1900 || _year    >                                  9999)) ||
       (_month   && (_month   <      1 || _month   >                                    12)) ||
       (_week    && (_week    <      1 || _week    >                                    53)) ||
       (_weekday && (_weekday <      0 || _weekday >                                     6)) ||
@@ -3413,7 +3424,7 @@ bool Datetime::leapYear (int year)
 int Datetime::daysInMonth (int year, int month)
 {
   // Protect against arguments being passed in the wrong order.
-  assert (year >= 1969 && year < 2100);
+  assert (year >= 1969 && year <= 9999);
   assert (month >= 1 && month <= 31);
 
   static int days[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
